"""
The huoguoml package provides a easy way for managing Machine Learning types and
services
"""

# DO NOT UPDATE THIS MANUALLY
__version__ = "1624103893"
__license__ = "Apache-2.0"

from huoguoml.tracking import start_experiment_run